@@ -9,7 +9,8 @@ from copy import deepcopy
 from io import BytesIO
 from pathlib import Path
 from shutil import copyfileobj
-from typing import Optional, List, Iterable, Dict, Union, Tuple
+from typing import Optional, List, Iterable, Union, Tuple
+from urllib.parse import urlparse
 
 import numpy as np
 import pandas as pd
@@ -189,7 +190,7 @@ def multipart_upload(local_path, companion_url):
     PART_SIZE = 5 * 1024 ** 2
     etags = []
     part = 0
-    with open(local_path, 'r') as f:
+    with open(local_path, 'rb') as f:
         while True:
             file_data = f.read(PART_SIZE)
             if not file_data:
@@ -996,8 +997,8 @@ class SMDataset(object):
         neutral_loss='',
         chem_mod='',
     ):
-        """
-        Retrieve ion images for a specific sf and adduct
+        """Retrieve ion images for a specific sf and adduct.
+
         :param str   sf:
         :param str   adduct:
         :param bool  only_first_isotope: Only retrieve the first (most abundant) isotopic ion image.
@@ -1027,10 +1028,15 @@ class SMDataset(object):
         def fetchImage(url):
             if not url:
                 return None
-            url = self._baseurl + url
+            if not urlparse(url).netloc:
+                # Ion images that have not been moved to S3 are sent as relative paths that need
+                # a host prefixed. This fallback can likely be removed after 2021-04-01.
+                url = self._baseurl + url
+
             try:
                 im = mpimg.imread(BytesIO(self._session.get(url).content))
             except:
+                # Retry, because occasionally the first request fails with a timeout
                 im = mpimg.imread(BytesIO(self._session.get(url).content))
             mask = im[:, :, 3]
             data = im[:, :, 0]
@@ -1051,14 +1057,15 @@ class SMDataset(object):
 
         if scale_intensity:
             non_empty_images = [i for i in images if i is not None]
-            shape = non_empty_images[0].shape
-            for i in range(len(images)):
-                if images[i] is None:
-                    images[i] = np.zeros(shape, dtype=non_empty_images[0].dtype)
-                else:
-                    lo = float(image_metadata[i]['minIntensity'])
-                    hi = float(image_metadata[i]['maxIntensity'])
-                    images[i] = lo + images[i] * (hi - lo)
+            if non_empty_images:
+                shape = non_empty_images[0].shape
+                for i in range(len(images)):
+                    if images[i] is None:
+                        images[i] = np.zeros(shape, dtype=non_empty_images[0].dtype)
+                    else:
+                        lo = float(image_metadata[i]['minIntensity'])
+                        hi = float(image_metadata[i]['maxIntensity'])
+                        images[i] = lo + images[i] * (hi - lo)
 
         if hotspot_clipping:
             for i in range(len(images)):
@@ -1124,7 +1131,12 @@ class SMDataset(object):
 
             if not url:
                 return None
-            url = self._baseurl + url
+
+            if not urlparse(url).netloc:
+                # Images that have not been moved to S3 are sent as relative paths that need
+                # a host prefixed. This fallback can be removed after
+                # https://github.com/metaspace2020/metaspace/issues/803 is implemented
+                url = self._baseurl + url
             im = Image.open(BytesIO(self._session.get(url).content))
             return np.asarray(im)
 
@@ -1144,8 +1156,8 @@ class SMDataset(object):
             return json.loads(download_link_json)
 
     def download_to_dir(self, path, base_name=None):
-        """
-        Downloads the dataset's input files to the specified directory
+        """Downloads the dataset's input files to the specified directory.
+
         :param path: Destination directory
         :param base_name: If specified, overrides the base name (excluding extension) of each file.
                           e.g. `base_name='foo'` will name the files as 'foo.imzML' and 'foo.ibd'
@@ -1206,6 +1218,8 @@ class MolecularDB:
 
 
 class SMInstance(object):
+    """Client class for communication with the Metaspace API."""
+
     def __init__(
         self,
         host: str = None,
@@ -1215,8 +1229,7 @@ class SMInstance(object):
         api_key: str = None,
         config_path: str = None,
     ):
-        """Client class for communication with the Metaspace API.
-
+        """
         Args:
             host: Full host name, e.g. 'https://metaspace2020.eu'
             verify_certificate: Ignore certificate validation.
@@ -1322,7 +1335,8 @@ class SMInstance(object):
             [
                 pd.DataFrame(json_normalize(json.loads(dataset['metadataJson'])))
                 for dataset in datasets
-            ]
+            ],
+            sort=False,
         )
         df.index = [dataset['id'] for dataset in datasets]
         return df
